#ifndef ATCHOPS_BASE64_H
#define ATCHOPS_BASE64_H
#ifdef __cplusplus
extern "C" {
#endif

#include <atchops/platform.h> // IWYU pragma: keep
#include <stddef.h>

/**
 * @brief Base64 encode some bytes
 *
 * @param src src bytes that you want to encode
 * @param srcsize the length of the src bytes
 * @param dst the buffer where the base64 encoded result will be
 * @param dstsize the buffer size, you can use atchops_base64_encoded_size to calculate the optimal size
 * @param dstlen the length of the result after operation, NULLable if you don't want the length
 * @return int 0 on success
 */
int atchops_base64_encode(const unsigned char *src, const size_t srcsize, char *dst, size_t dstsize, size_t *dstlen);

/**
 * @brief Base64 decode some bytes
 *
 * @param src src bytes that you want to decode
 * @param srcsize the length of the src bytes
 * @param dst the buffer where the base64 decoded result will be
 * @param dstsize the buffer size, you may use atchops_base64_decoded_size to calculate the buffer size
 * @param dstlen the length of the result after operation, NULLable if you don't want the length
 * @return int 0 on success
 */
int atchops_base64_decode(const char *src, const size_t srcsize, unsigned char *dst, size_t dstsize, size_t *dstlen);

/**
 * @brief calculate the size of the base64 encoded string. This function is usually called before encoding to allocate
 * the optimal buffer size
 *
 * @param plaintextsize the size of the original plain text (to be encoded)
 * @return size_t the output buffer size needed to store the encoded base64 string
 */
size_t atchops_base64_encoded_size(const size_t plaintextsize);

/**
 * @brief calculate the size of the base64 decoded string. This function is usually called before decoding to allocate
 * the optimal buffer size
 *
 * @param encodedsize the size of the base64 encoded string (to be decoded)
 * @return size_t the output buffer size needed to store the decoded plain text
 */
size_t atchops_base64_decoded_size(const size_t encodedsize);

#ifdef __cplusplus
}
#endif
#endif
